#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include "common.h"

/*--------------------------------------------------------------------*/

#define MAX_DOWNLOAD 128

int download (void);

/*--------------------------------------------------------------------*/

int cgiMain (void)
{
    char   *p, **pp, buf[32], *f_dir, *f_name;
    struct file fs[MAX_DOWNLOAD];
    int    N, i;
    unsigned long total_size;

    get_options ();

    cgiFormString ("act", buf, sizeof (buf));
    if (strcmp (buf, " Start download ") == 0)
        return download ();
    
    if (load_database ())
        error ("Can't load list of files");

    if (cgiFormStringMultiple ("d", &pp) != cgiFormSuccess)
        error ("Nothing was selected to download");

    cgiHeaderContentType ("text/html");
    fprintf (cgiOut, HTML_HEADER, "Download");
    print_file (HEADER);

    fprintf (cgiOut, "\n<h2>Download</h2>");
        
    // build list of files selected for downloading
    N = 0;
    while (*pp != NULL)
    {
        // record it into array if the file does exist
        if (access (make_absolute (*pp), 0) == 0)
        {
            p = strrchr (*pp, '/');
            if (p != NULL)
            {
                *p = '\0';
                f_dir  = *pp;
                f_name = p+1;
                for (i=0; i<fb.N; i++)
                    if (!strcmp (fb.files[i].dir, f_dir) &&
                        !strcmp (fb.files[i].name, f_name))
                    {
                        fs[N] = fb.files[i];
                        N++;
                        break;
                    }
            }
        }
        pp++;
    }

    total_size = 0;
    for (i=0; i<N; i++)
        total_size += fs[i].size;
    
    // print list of files (allow user to select/unselect them)
    // print download form
    fprintf (cgiOut, "<form action=\""AUTODOWNLOAD1"\" method=post>"
            "\n<pre>");
    print_list (fs, N, LIST_ALL_CHECKED|LIST_SINGLE_DIR);
    fprintf (cgiOut, "\n</pre>");
    fprintf (cgiOut, "<hr>"
            "\nYou have selected <b>%d</b> files, <b>%s</b> bytes total.<br>\n"
            "\nSelect method of archiving:<b>\n"
            "\n<input type=radio name=arcm value=1 %s> ZIP or \n"
            "\n<input type=radio name=arcm value=2 %s> TAR "
            "</b>\n",
            N, insert_commas (total_size),
            options.arc_method == 1 ? "checked" : "",
            options.arc_method == 2 ? "checked" : "");
    put_options_form (0);
    fprintf (cgiOut, "\n<input type=submit value=Refresh>"
            "\n<input type=submit name=act value=\" Start download \">"
            "\n</form>");
    
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    return 0;
}

/* ------------------------------------------------------------------ */

static char *ext[] = {"", "zip", "tar"};

int download (void)
{
    char   **pp, **pp1, *p, temp_list[4096], temp_arc[4096], buffer[4096];
    char   command[16384];
    FILE   *F_temp_list;

    get_options ();

    if (options.arc_method != 1 && options.arc_method != 2)
        error ("Wrong archiving type");

    if (cgiFormStringMultiple ("d", &pp) != cgiFormSuccess)
        error ("Nothing was selected to download");

    // form names of temporary files
    sprintf (temp_list, "/tmp/list%05d", getpid ());
    sprintf (temp_arc, ARCHIVE_PATH"scratch/arc%05d.%s", getpid (),
             ext[options.arc_method]);

    // build list of files selected for downloading
    pp1 = pp;
    if (options.arc_method == 1)
    {
        F_temp_list = fopen (temp_list, "w");
        while (*pp1 != NULL)
        {
            sprintf (buffer, "%s%s", FTP_ROOT, *pp1);
            if (access (buffer, 00) == 0)    // skip files which do not exist
                fprintf (F_temp_list, "\n%s%s", FTP_ROOT, *pp1);
            pp1++;
        }
        fclose (F_temp_list);
    }

    // create archives
    if (options.arc_method == 1)
    {
        sprintf (command, "zip -1 -j -@ %s <%s 1>"NULL_DEV" 2>"NULL_DEV,
                 temp_arc, temp_list);
        cgiSaferSystem (command);
    }
    if (options.arc_method == 2)
    {
        // create archive first
        sprintf (command, "tar cf %s 1>"NULL_DEV" 2>"NULL_DEV,
                 temp_arc);
        cgiSaferSystem (command);
        // add files file-by-file
        pp1 = pp;
        while (*pp1 != NULL)
        {
            sprintf (buffer, "%s%s", FTP_ROOT, *pp1);
            p = strrchr (buffer, '/');
            // skip files which do not exist
            if (access (buffer, 00) == 0 && p != NULL)
            {
                *p = '\0';
                sprintf (command, "tar rf %s -C %s %s 1>"NULL_DEV" 2>"NULL_DEV,
                         temp_arc, buffer, p+1);
                cgiSaferSystem (command);
            }
            pp1++;
        }
    }

    // remove list
    remove (temp_list);

    // send file via HTTP
    fprintf (cgiOut, "Location: http://"
             ARCHIVE_HOST ARCHIVE_URL "/scratch/arc%05d.%s\n",
             getpid(),
             ext[options.arc_method]);

    // put a message for older browsers
    cgiHeaderContentType ("text/html");
    fprintf (cgiOut, HTML_HEADER, "your file");
    print_file (HEADER);

    fprintf (cgiOut,
             "<h1>Your file</h1>\n"
             "<a href=\"http://"ARCHIVE_HOST ARCHIVE_URL "/scratch/arc%05d.%s\">Download</a> it...",
             getpid(), ext[options.arc_method]);
             
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    
    /*
    fh = open (temp_arc, O_RDONLY);
    if (fh > 0)
    {
        fprintf (cgiOut, "Content-Length: %ld\n", file_length (fh));
        close (fh);
        }
    if (options.arc_method == 1)
        cgiHeaderContentType ("application/zip");
    if (options.arc_method == 2)
        cgiHeaderContentType ("application/tar");
    sprintf (command, "cat %s", temp_arc);

    fflush (cgiOut);
    cgiSaferSystem (command);
        */

    // remove list
    remove (temp_list);
    
    return 0;
}
