#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include "common.h"

/*--------------------------------------------------------------------*/

int cgiMain (void)
{
    int     i, M = 0, M1 = 0, x;
    struct file *fs;
    time_t now, age;

    if (load_database ()) error ("Can't load list of files");

    // output HTTP and HTML headers which are always needed
    cgiHeaderContentType ("text/html");
    
    fprintf (cgiOut, HTML_HEADER, "new files");
    print_file (HEADER);

    // parse CGI parameters
    options.sort.default_type = 't';
    get_options ();
    
    // extract search information from parameters
    age = 0;
    cgiFormIntegerBounded ("age_d", &x, 0, 10000, 7);
    age += x*24*60;
    cgiFormIntegerBounded ("age_h", &x, 0, 24, 0);
    age += x*60;
    cgiFormIntegerBounded ("age_m", &x, 0, 60, 0);
    age += x;
    if (age == 0) age = 7*24*60; // last seven days
    
    // print form etc.
    fprintf (cgiOut, "\n<h2>New files for last %ld days, %ld hours, %ld minutes</h2>\n",
             age/(24*60), (age/60)%24, age%60);
    fprintf (cgiOut, "\n<form method=get action=\""AUTONEWFILES"\">\n\n"
             "<center><table border=0 cellpadding=5 align=center>"
             "\n<tr>\n"
             "<td align=center colspan=2>Enter time to count backwards and press Enter:\n"
             "\n<tr>\n"
             "<td nowrap><input name=age_d size=5 value=\"%ld\"> days"
             "<td nowrap><input name=age_h size=5 value=\"%ld\"> hours"
             "<td nowrap><input name=age_m size=5 value=\"%ld\"> minutes"
             "<td nowrap><input type=submit value=\"Display\">\n"
             "(<a href=\"help.html#newfiles\">help</a>)"
             "</table>\n",
             age/(24*60), (age/60)%24, age%60);
    fprintf (cgiOut, "<center>\n"
             "Search\n"
             "<input type=radio name=opt5 value=0 %s> entire archive\n"
             "or <input type=radio name=opt5 value=1 %s> under\n"
             "<br><input type=text name=path size=50 value=\"%s\">"
             "</center>",
             options.search.from_subdir ? "" : "checked",
             options.search.from_subdir ? "checked" : "",
             options.path);
    fprintf (cgiOut, "<table border=0 cellpadding=5 align=center>\n"
             "\n<tr>\n"
             "<td>Sort by:\n"
             "<td nowrap><input name=sort type=radio value=n %s> Name\n"
             "(<input name=sort type=radio value=N %s> reversed)\n"
             "<input name=sort type=radio value=s %s> Size\n"
             "(<input name=sort type=radio value=S %s> reversed)\n"
             "<input name=sort type=radio value=t %s> Date\n"
             "(<input name=sort type=radio value=T %s> reversed)\n"
             "\n<tr>\n"
             "<td>Options:\n"
             "<td>\n"
             "<input type=checkbox name=opt3 value=1 %s> Sort by directories\n"
             "<input type=checkbox name=opt4 value=1 %s> Print directory names\n"
             "</table></center>",
             options.sort.type == 'n' ? "checked" : "",
             options.sort.type == 'N' ? "checked" : "",
             options.sort.type == 's' ? "checked" : "",
             options.sort.type == 'S' ? "checked" : "",
             options.sort.type == 't' ? "checked" : "",
             options.sort.type == 'T' ? "checked" : "",
             options.sort.by_directory ? "checked" : "",
             options.search.print_where ? "checked" : ""
            );
    put_options_form (SKIP_SORT_TYPE|SKIP_SORT_BYDIR|SKIP_PATH|
                      SKIP_SRCH_PRINTWHERE|SKIP_SRCH_FROMSUBDIR);
    fprintf (cgiOut, "</form>\n\n");

    // allocate space for found files
    M = 0;
    fs = malloc (fb.N * sizeof (struct file));

    // query current time
    now = time (NULL);
    fprintf (cgiOut, "Search database was updated %ld hour(s), %ld minutes ago.\n",
           (now-fb.timestamp)/(60*60), ((now-fb.timestamp)/60)%60);

    // select files within requested period but not into the future
    for (i=0; i<fb.N; i++)
    {
        if (fb.files[i].type == TYPE_REGULAR && // regular file
            in_the_search_scope (i, options.path))
        {
            if (fb.files[i].age > now-age*60 &&     // younger than age
                fb.files[i].age < now)              // ignore files from the future
            {
                fs[M] = fb.files[i];
                M++;
            }
            M1++;
        }
    }

    if (M > 10)
        if (options.group_selection)
        {
            fprintf (cgiOut, "\n<br><a href=\""AUTONEWFILES"?%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        }
        else
        {
            fprintf (cgiOut, "\n<br><a href=\""AUTONEWFILES"?%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        }
    
    // print statistics on search
    fprintf (cgiOut, "\n<hr>\n\n<b>%d</b> file(s) are younger than "
            "%ld days, %ld hours, %ld minutes among %d files\n",
            M, age/(24*60), (age/60)%24, age%60, M1);
    if (M == 0) goto finish;
    
    // sort and print them
    qsort ((void *)fs, M, sizeof (struct file), filecompare);
    
    // print download form
    if (options.group_selection)
        fprintf (cgiOut, "<form action=\""AUTODOWNLOAD1"\" method=post>");
    fprintf (cgiOut, "<pre>");
    print_list (fs, M, 0);
    fprintf (cgiOut, "\n</pre>");
    if (options.group_selection)
    {
        put_options_form (0);
        fprintf (cgiOut, "\n<input type=submit value=Download>"
                "\n</form>");
    }

    if (M > 0)
        if (options.group_selection)
        {
            fprintf (cgiOut, "\n<a href=\""AUTONEWFILES"?%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        }
        else
        {
            fprintf (cgiOut, "\n<a href=\""AUTONEWFILES"?%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        }

finish:
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    
    return 0;
}

