#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include "common.h"

/*--------------------------------------------------------------------*/

static int do_search (int n, char *s);

int cgiMain (void)
{
    int     i, M = 0, M1 = 0;
    char    searchstring[4096], typestring[2];
    struct  file *fs;
    time_t  now;

    if (load_database ()) error ("Can't load list of files");

    // output HTTP and HTML headers which are always needed
    cgiHeaderContentType ("text/html");
    
    fprintf (cgiOut, HTML_HEADER, "search");
    print_file (HEADER);

    // parse CGI parameters
    get_options ();

    // extract search information from parameters
    cgiFormString ("type", typestring, sizeof (typestring));
    if (!strcmp (typestring, "p"))
        options.search.type = SEARCH_PLAIN;
    if (!strcmp (typestring, "w"))
        options.search.type = SEARCH_MASK;

    cgiFormStringNoNewlines ("target", searchstring, sizeof (searchstring));
    
    // print search form etc.
    if (searchstring == NULL || searchstring[0] == '\0')
        fprintf (cgiOut, "\n<h2>Search</h2>\n");
    else
        fprintf (cgiOut, "\n<h2>Search for [%s]</h2>\n", searchstring);
    fprintf (cgiOut, "\n<form method=get action=\""AUTOSEARCH"\">\n\n"
             "<center><table border=0 cellpadding=5 align=center>"
             "\n<tr>\n"
             "<td align=center colspan=2>Enter search word, and press Enter:\n"
             "\n<tr>\n"
             "<td nowrap><input name=target size=50 value=\"%s\">"
             "<td nowrap><input type=submit value=\"Search\">\n"
             "(<a href=\"help.html#search\">help</a>)"
             "</table>\n",
             searchstring == NULL ? "" : searchstring);
    fprintf (cgiOut, "<center>\n"
             "Search\n"
             "<input type=radio name=opt5 value=0 %s> entire archive\n"
             "or <input type=radio name=opt5 value=1 %s> under\n"
             "<br><input type=text name=path size=50 value=\"%s\">"
             "</center>",
             options.search.from_subdir ? "" : "checked",
             options.search.from_subdir ? "checked" : "",
             options.path);
    fprintf (cgiOut, "<table border=0 cellpadding=5 align=center>\n"
             "\n<tr>\n"
             "<td>Search type:\n"
             "<td nowrap><input name=type type=radio value=p %s> Plain\n"
             "<input name=type type=radio value=w %s> Wildcard\n"
             "\n<tr>\n"
             "<td>Sort by:\n"
             "<td nowrap><input name=sort type=radio value=n %s> Name\n"
             "(<input name=sort type=radio value=N %s> reversed)\n"
             "<input name=sort type=radio value=s %s> Size\n"
             "(<input name=sort type=radio value=S %s> reversed)\n"
             "<input name=sort type=radio value=t %s> Date\n"
             "(<input name=sort type=radio value=T %s> reversed)\n"
             "\n<tr>\n"
             "<td>Search options:\n"
             "<td>\n"
             "<input type=checkbox name=opt1 value=1 %s> Case sensitive\n"
             "<input type=checkbox name=opt2 value=1 %s> Exclude directory names (plain search only)\n"
             "\n<tr>\n"
             "<td>\n"
             "<td>\n"
             "<input type=checkbox name=opt3 value=1 %s> Sort by directories\n"
             "<input type=checkbox name=opt4 value=1 %s> Print directory names\n"
             "</table></center>\n",
             options.search.type == SEARCH_PLAIN ? "checked" : "",
             options.search.type == SEARCH_MASK ? "checked" : "",
             options.sort.type == 'n' ? "checked" : "",
             options.sort.type == 'N' ? "checked" : "",
             options.sort.type == 's' ? "checked" : "",
             options.sort.type == 'S' ? "checked" : "",
             options.sort.type == 't' ? "checked" : "",
             options.sort.type == 'T' ? "checked" : "",
             options.search.casesen ? "checked" : "",
             options.search.exclude_dirnames ? "checked" : "",
             options.sort.by_directory ? "checked" : "",
             options.search.print_where ? "checked" : ""
            );
    put_options_form (SKIP_SORT_TYPE|SKIP_SRCH_CASESEN|SKIP_SRCH_NODIRS
                      |SKIP_PATH|SKIP_SRCH_PRINTWHERE|SKIP_SRCH_FROMSUBDIR);
    fprintf (cgiOut, "</form>\n\n");

    // check whether search string was specified
    if (searchstring == NULL || *searchstring == '\0') goto finish;

    now = time (NULL);
    fprintf (cgiOut, "Search database was updated %ld hours, %ld minutes ago.\n",
           (now-fb.timestamp)/(60*60), ((now-fb.timestamp)/60)%60);
    
    // search for specified string
    fs = malloc (fb.N * sizeof (struct file));
    for (i=0; i<fb.N; i++)
    {
        if (fb.files[i].type == TYPE_REGULAR &&
           in_the_search_scope (i, options.path))
        {
            if (do_search (i, searchstring))
            {
                fs[M] = fb.files[i];
                M++;
            }
            M1++;
        }
    }

    fprintf (cgiOut, "\n<hr>\n\n<b>%d</b> file(s) match specified "
             "string [<b>%s</b>] among %d files\n", M, searchstring, M1);
    if (M == 0) goto finish;
    
    if (M > 10)
        if (options.group_selection)
        {
            fprintf (cgiOut, "\n<br><a href=\""AUTOSEARCH"?%s&target=%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL), hexify (searchstring));
        }
        else
        {
            fprintf (cgiOut, "\n<br><a href=\""AUTOSEARCH"?%s&target=%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL), hexify (searchstring));
        }

    // sort and print them
    qsort ((void *)fs, M, sizeof (struct file), filecompare);
    
    // print download form
    if (options.group_selection)
        fprintf (cgiOut, "<form action=\""AUTODOWNLOAD1"\" method=post>");
    fprintf (cgiOut, "<pre>");
    print_list (fs, M, 0);
    fprintf (cgiOut, "\n</pre>");
    if (options.group_selection)
    {
        put_options_form (0);
        fprintf (cgiOut, "\n<input type=submit value=Download>"
                "\n</form>");
    }

    if (M > 0)
        if (options.group_selection)
        {
            fprintf (cgiOut, "\n<a href=\""AUTOSEARCH"?%s&target=%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL), hexify (searchstring));
        }
        else
        {
            fprintf (cgiOut, "\n<a href=\""AUTOSEARCH"?%s&target=%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL), hexify (searchstring));
        }

finish:
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    
    return 0;
}

/* ----------------------------------------------------------------- */

static int do_search (int n, char *s)
{
    int    result;
    char   pname[4096];
    
    switch (options.search.type)
    {
    case SEARCH_PLAIN:
        if (options.search.exclude_dirnames)
            strcpy (pname, fb.files[n].name);
        else
        {
            strcpy (pname, fb.files[n].dir);
            strcats (pname, fb.files[n].name);
        }
        if (options.search.casesen)
            result = (strstr (pname, s) != NULL);
        else
            result = (stristr (pname, s) != NULL);
        if (!result && fb.files[n].desc != NULL)
            result = (stristr (fb.files[n].desc, s) != NULL);
        break;
    case SEARCH_MASK:
        strcpy (pname, fb.files[n].name);
        if (options.search.casesen)
            result = mask_match (s, pname);
        else
            result = mask_matchi (s, pname);
        break;
    default:
        result = 0;
    }
    return result;
}
