#ifndef __COMMON_H_INCLUDED
#define __COMMON_H_INCLUDED

#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include "cgic.h"
#include "tools1.h"
#include "tools2.h"
#include "filebase2.h"
#include "onsite.h"

/* some useful strings */

#define HTML_HEADER   "<html><head>\n\n" \
                      "<title> " ARCHIVE_NAME " : %s</title>\n" \
                      "\n</head><body>\n\n"
#define HTTP_FOOTER    "\n\n</body></html>"
#define ERROR_REPORT "\n<br>Please <a href=\"mailto:"MAINTAINER"\">submit error report</a> to archive manager"
#define ERROR_MSG    "<hr><b>Error happened while processing:</b> %s.\n<p>\nReturn to <a href=\""ARCHIVE_URL"\">archive homepage</a>\n"ERROR_REPORT

/* constants and definitions */

#define SEARCH_PLAIN   1
#define SEARCH_MASK    2
#define SEARCH_REGEXP  3

/* flags for print_list */

#define LIST_ALL_CHECKED    0x0001
#define LIST_SINGLE_DIR     0x0002

/* flags for put_options/put_options_form */

#define SKIP_SORT_TYPE           0x0001
#define SKIP_SORT_BYDIR          0x0002
#define SKIP_SRCH_CASESEN        0x0010
#define SKIP_SRCH_NODIRS         0x0020
#define SKIP_SRCH_PRINTWHERE     0x0040
#define SKIP_SRCH_FROMSUBDIR     0x0080
#define SKIP_TREE_PRINTDESC      0x1000
#define SKIP_TREE_DEPTH          0x2000
#define SKIP_GROUPSEL            0x4000
#define SKIP_ARCMETHOD           0x8000
#define SKIP_PATH                0x0100

/* useful macros */

#define HEX2DEC(h)  ((h) < '0' ? 0 : (h) <= '9' ? (h)-'0' : \
(h) < 'A' ? 0 : (h) <= 'F' ? (h)-'A'+10 : \
    (h) < 'a' ? 0 : (h) <= 'f' ? (h)-'a'+10 : 0)
    
/* Types */

struct __options
{
    struct __search
    {
        int    type;
        int    casesen;
        int    exclude_dirnames;
        int    print_where;
        int    from_subdir;
    }
    search;
    struct __sort
    {
        int    type;
        int    by_directory;
        int    default_type;
    }
    sort;
    struct __tree
    {
        int   max_depth;
        int   print_desc;
    }
    tree;
    int   group_selection;
    int   arc_method;    /* 1 - zip; 2 - tar */
    char  path[4096];
};

extern struct __options options;
extern char *month[12];
/*extern char *QUERY_STRING, *DOCUMENT_ROOT;*/

/* Function definitions */

int  filecompare (__const__ void *elem1, __const__ void *elem2);
/*void cgi_init (void);*/
/*char *extract_variable (char *varname);*/
int  is_validsort (void);
void print_directory_name (char *dirname, char *desc);
void print_sort_urls (char *url);
void print_list (struct file *f, int N, int flags);
void get_options (void);
char *put_options (int skip);
void put_options_form (int skip);
void print_footer (void);
int  in_the_search_scope (int n, char *startpath);
void error (char *format, ...);

#endif
